module defineparameters
    implicit none
    !This module defines the parameters of the program
    !The values of calibrated parameters are set in the main file
    !n is number of grid points for capital, p is the number of grid points for human capital.
    !q is the number of abilities, l is the number of income shocks
    !beta is the discount factor
    !r is the risk free interest rate
    !eta is a preference parameter (inverse of Frisch elasticity)
    !gamma 1-3 determnines return to experience and human capital
    !wmin is the minimum wage
    
    integer, parameter :: n=40, p=16, q=13, l=7
    real(8) :: beta=0.96, r=0.0609, sigma=2.0, lumpsum=0.043, lumpsumdum=0.043
    real(8), parameter :: eta=3.0, gam1=0.220537, gam2=-0.0292002, gam3=0.0010142
    real(8), parameter :: gamc1=0.2945845, gamc2=-0.0515872, gamc3=0.0030048
    real(8), parameter :: pi=3.14159265, wmin=0.25
    
    !gk, gh, ghc will contain the policy rule for capital, human capital, and child's human capital. (there are many versions of these arrays because the program was once run on a small computer with insufficient memory)
    !gn, gb are the policy functions for labor supply and gift to the child
    !transu and transa are the transition matrixes for earnings shocks and abilities. 
    !kmin, kmax, hmin, hmax, bound the spaces for capital and human capital.
    !U, A are the vectors of earnings shocks and abilities
    !K, H contains the grid for capital and human capital
    !COEFS holds coefficients for interpolation of the value function
    !Random hold random numbers used in the interpolation
    !Xs holds stored values from the simulation
    
    real(8), dimension (:,:,:,:,:,:), allocatable :: gk1,gn1,gb,c1,temp2,Vone,temp3,gk2,gn2,c2,temp22,Vtwo,col,gk3,gn3,c3,temp23,Vthree,gk4,gh4,gn4,c4,temp24,Vfour,inv4
    real(8), dimension (:,:,:,:,:,:), allocatable :: gk5,gh5,gn5,c5,c6,c7,c8,c9,Vfive,temp25,inv5,gk6,gh6,gn6,Vsix,temp26,inv6,gk7,gn7,Vseven,temp27,gk8,gn8,Veight,temp28,gk9,gh9,gn9,Vnine,temp29,inv9
    real(8), dimension (1:2,1:7,1:n,1:p,1:p,1:q,1:l) :: gko, co, Vo, temp3o
    real(8), dimension (1:2,1:n,1:p,1:p,1:q,1:l) :: measure, measure2
    real(8), dimension (1:56,9,2,q,l) :: COEFS
    real(8), dimension (1:640,9,p,2,q,l) :: COEFS2
    real(8), dimension (1:38,9,2,q,l) :: COEFS3
    real(8), dimension (1:40,9,p,2,q,l) :: COEFS4
    real(8), dimension (1:40,16,p,2,7,l) :: COEFS5
    real(8), dimension (1:2000000,1:22) :: Random
    real(8), dimension (1:200000) :: Random2
    real(8), dimension (1:2,1:l,1:l) :: transu
    real(8), dimension (1:q,1:q) :: transa
    real(8), dimension (1:n) :: K
    real(8), dimension (1:p) :: H
    real(8), dimension (1:q) :: A, proba
    real(8), dimension (1:2,1:l) :: U, probu
    real(8), dimension (1:200000,1:133) :: XS
    integer, dimension (1:200000,1:2) :: XS2

    !alpha is the altruism parameter
    !psi governs the production of human capital
    !chi governs the disutility of labor
    !ma is average ability (normalized to 1, so practically redundant)
    !hs is the starting value of human capital. kmin,kmax,hmin,hmax defines the sates space for capital and human capital
    !omega governs the probability of college failure
    !sigu,siga,rhoa,rhou govern the processes for abilities and idiosyncratic shocks
    !AE is average earnings
    
    real(8) :: psi, psi2, psi3, psi4, gam0, gamc0, chi, zet, ma,alpha, siga, rhoa, sigu, rhou, omega,t1,t2,t3,t4
    real(8) :: kmin, kmax, hmin, hs, hmax, inck, epsilon=0.1, epsilon2=0.1, AE=0.3105, Aedum
    
    integer, dimension (1:4) :: sim
    integer :: i,d2,loopcount
    
    !US Tax
    real(8), parameter :: t1m=-0.5952387, t2m=1.636844, t3m=-1.007828, t4m=0.1970667, tc=0.084
    real(8), parameter :: t1mc=-1.512787, t2mc=3.473651, t3mc=-2.235034, t4mc=0.4697056
    real(8), parameter :: t1s=-1.18305, t2s=3.181103, t3s=-2.252743, t4s=0.5132841
    
    !Danish Tax
    !real(8), parameter :: t1m=-0.8340821, t2m=2.653975, t3m=-1.782215, t4m=0.3727709, tc=0.25
    !real(8), parameter :: t1mc=-2.737235, t2mc=6.483092, t3mc=-4.328906, t4mc=0.9324725
    !real(8), parameter :: t1s=-1.851625, t2s=4.995183, t3s=-3.491631, t4s=0.7754541
    
    !Norwegian tax
    !real(8), parameter :: t1m=-0.9151925, t2m=2.357054, t3m=-1.436, t4m=0.2768166, tc=0.237
    !real(8), parameter :: t1mc=-1.687183, t2mc=3.881035, t3mc=-2.436887, t4mc=0.4949357
    !real(8), parameter :: t1s=-0.9186265, t2s=2.453045, t3s=-1.511725, t4s=0.2899294
    
    !Finish tax
    !real(8), parameter :: t1m=-1.786036, t2m=4.210806, t3m=-2.709563, t4m=0.5645917, tc=0.22
    !real(8), parameter :: t1mc=-3.258404, t2mc=7.108649, t3mc=-4.592615, t4mc=0.9685161
    !real(8), parameter :: t1s=-2.314598, t2s=5.579046, t3s=-3.775753, t4s=0.826796
    
    !Spanish tax
    !real(8), parameter :: t1m=-0.695431, t2m=1.482619, t3m=-0.724988, t4m=0.1047873, tc=0.16
    !real(8), parameter :: t1mc=-0.8535222, t2mc=1.800009, t3mc=-0.9442255, t4mc=0.1558268
    !real(8), parameter :: t1s=-0.7455719, t2s=1.710186, t3s=-0.924621, t4s=0.155641
    
    !French tax
    !real(8), parameter :: t1m=-0.0659207, t2m=0.5232531, t3m=-0.2436658, t4m=0.0372464, tc=0.199
    !real(8), parameter :: t1mc=0.145302, t2mc=-0.2255823, t3mc=0.4825532, t4mc=-0.1739126
    !real(8), parameter :: t1s=-0.6403266, t2s=1.995908, t3s=-1.385199, t4s=0.3145717
    
    !German tax
    !real(8), parameter :: t1m=-2.341808, t2m=6.033008, t3m=-4.321287, t4m=0.9888285, tc=0.16
    !real(8), parameter :: t1mc=-2.831994, t2mc=6.707468, t3mc=-4.575432, t4mc=1.004294
    !real(8), parameter :: t1s=-1.278933, t2s=3.923879, t3s=-2.909065, t4s=0.6716755
    
    !Canadian tax
    !real(8), parameter :: t1m=-1.004608, t2m=2.383766, t3m=-1.467924, t4m=0.2944103, tc=0.07
    !real(8), parameter :: t1mc=-3.044256, t2mc=6.512615, t3mc=-4.210616, t4mc=0.8934768
    !real(8), parameter :: t1s=-0.3056732, t2s=0.8059581, t3s=-0.2546371, t4s=-0.0145851
    
    !Italian tax
    !real(8), parameter :: t1m=-2.199901, t2m=5.172168, t3m=-3.501398, t4m=0.7705837, tc=0.2
    !real(8), parameter :: t1mc=-2.972545, t2mc=6.547168, t3mc=-4.289763, t4mc=0.9156697
    !real(8), parameter :: t1s=-2.33939, t2s=5.629314, t3s=-3.884271, t4s=0.8666075
    
    !UK tax
    !real(8), parameter :: t1m=-1.751862, t2m=4.313386, t3m=-3.016562, t4m=0.6846306, tc=0.175
    !real(8), parameter :: t1mc=-3.387389, t2mc=7.400429, t3mc=-4.917345, t4mc=1.06676
    !real(8), parameter :: t1s=-1.815518, t2s=4.588508, t3s=-3.26912, t4s=0.7516708
    
    !Swedish tax
    !real(8), parameter :: t1m=0.0435576, t2m=0.2558834, t3m=0.117199, t4m=-0.1021613, tc=0.25
    !real(8), parameter :: t1mc=-1.89896, t2mc=4.382775, t3mc=-2.78672, t4mc=0.5725898
    !real(8), parameter :: t1s=-0.8620883, t2s=2.485298, t3s=-1.601609, t4s=0.3224937
    
    
    end module defineparameters