INCLUDE 'link_fnl_static.h'
!DEC$ OBJCOMMENT LIB:'libiomp5md.lib'
      
! Add this source to your project to correct the problem
! that, for a Debug build for certain projects using the
! IMSL Fortran libraries, the linker complains of an
! unresolved external symbol __imp__MPIPRIV
BLOCK DATA MPIPRIV_DEF
COMMON /MPIPRIV/ DUMMY
!DEC$ ATTRIBUTES ALIAS:"__imp__MPIPRIV" :: /MPIPRIV/
END BLOCK DATA MPIPRIV_DEF


!****************************************************************************
!
!  PROGRAM: IGPmodel.f90
!
!  Subroutines:
!  Define parameters, defines all the parameters used in the program
!  Transmatrix - Fills the transition matrix and the spaces for human and physical capital
!  Max1to5solves the problem of retired people
!  Max4, Max5....Max12 solves the problems of people aged 60-20
!  FCN6 contains the maximization problem at age 50 (to be called by max6)
!  FCN7 contains the maximization problem of agent aged 35, 40, 45 (to be called by max max9,max8,max7)
!  FCN12 contains the maximization problem of agents going to college (to be called by max12)
!  Regression, Regression2 etc. are used for computing coefficients for chebyshev collocation
!  Simulation runs the model simulation after computing optimal policies
!  Statistics computes various model statistics, using the data from the model simulation
!
!****************************************************************************

   program IGPmodel
    
    !NEQNF is the standard equation solver in the IMSL Library
    !define parameter is a module with parameter definitions
    
    USE LCONF_INT
    USE BCPOL_INT
    USE defineparameters
    
    implicit none
    
    integer :: j,it,it2,it3,it4,it5,it6,it7,it8,it9,temp,count,count2,count3,count4,exper
    integer :: d,d3,d4,ind,d5,d6,d1,MAXFCN
    real(8) :: wage,wage2,netinc,hours,dum,dum2,dum3,L1,L2,L3,dum4,dum5,diff,ip,dum6, dum7, dum8, dum9,labor1,labor2,dum1
    real(8):: SOL(3), XGUESS(3), XLB(3), XUB(3), GRAD(1,1), RHS(1), ACC=0.00000001, OBJ, FVALUE, FTOL
    integer, dimension (1:6) :: loc
    
    allocate(gk1(2,n,p,p,q,l))
    allocate(gn1(2,n,p,p,q,l))
    allocate(gb(2,n,p,p,q,l))
    allocate(c1(2,n,p,p,q,l))
    allocate(temp2(2,n,p,p,q,l))
    allocate(Vone(2,n,p,p,q,l))
    allocate(temp3(2,n,p,p,q,l))
    allocate(gk2(2,n,p,p,q,l))
    allocate(gn2(2,n,p,p,q,l))
    allocate(c2(2,n,p,p,q,l))
    allocate(temp22(2,n,p,p,q,l))
    allocate(Vtwo(2,n,p,p,q,l))
    allocate(col(2,n,p,p,q,l))
    allocate(gk3(2,n,p,p,q,l))
    allocate(gn3(2,n,p,p,q,l))
    allocate(c3(2,n,p,p,q,l))
    allocate(temp23(2,n,p,p,q,l))
    allocate(Vthree(2,n,p,p,q,l))
    allocate(gk4(2,n,p,p,q,l))
    allocate(gh4(2,n,p,p,q,l))
    allocate(gn4(2,n,p,p,q,l))
    allocate(c4(2,n,p,p,q,l))
    allocate(temp24(2,n,p,p,q,l))
    allocate(Vfour(2,n,p,p,q,l))
    allocate(inv4(2,n,p,p,q,l))
    allocate(gk5(2,n,p,p,q,l))
    allocate(gh5(2,n,p,p,q,l))
    allocate(gn5(2,n,p,p,q,l))
    allocate(c5(2,n,p,p,q,l))
    allocate(c6(2,n,p,p,q,l))
    allocate(c8(2,n,p,p,q,l))
    allocate(c7(2,n,p,p,q,l))
    allocate(c9(2,n,p,p,q,l))
    allocate(Vfive(2,n,p,p,q,l))
    allocate(temp25(2,n,p,p,q,l))
    allocate(inv5(2,n,p,p,q,l))
    allocate(gk6(2,n,p,p,q,l))
    allocate(gh6(2,n,p,p,q,l))
    allocate(gn6(2,n,p,p,q,l))
    allocate(Vsix(2,n,p,p,q,l))
    allocate(temp26(2,n,p,p,q,l))
    allocate(inv6(2,n,p,p,q,l))
    allocate(gk7(2,n,p,p,q,l))
    allocate(gn7(2,n,p,p,q,l))
    allocate(Vseven(2,n,p,p,q,l))
    allocate(temp27(2,n,p,p,q,l))
    allocate(gk8(2,n,p,p,q,l))
    allocate(gn8(2,n,p,p,q,l))
    allocate(Veight(2,n,p,p,q,l))
    allocate(temp28(2,n,p,p,q,l))
    allocate(gk9(2,n,p,p,q,l))
    allocate(gh9(2,n,p,p,q,l))
    allocate(gn9(2,n,p,p,q,l))
    allocate(Vnine(2,n,p,p,q,l))
    allocate(temp29(2,n,p,p,q,l))
    allocate(inv9(2,n,p,p,q,l))
    
!Reading in random numbers that will later be used in the simulation    
    
    open(1, file='random.txt', status='old')
    
        read (1, *) random
        
    close(2,status='save')
    
    open(1, file='random2.txt', status='old')
    
        read (1, *) random2
        
    close(2,status='save')

!Reading in an initial guess for the value and policy functions at age 20

   open(1, file='policy16.txt', status='old')
        
        read (1, *) COEFS4
             
   close(2,status='save')
   
   open(1, file='policy.txt', status='old')
        
        read (1, *) gk9
             
   close(2,status='save')
   
   open(1, file='policy2.txt', status='old')
        
        read (1, *) inv9
             
   close(2,status='save')
    
   
 epsilon=1d0   
    

        ma=1.0
        zet=-0.04074
        psi=0.44
        psi2=0.44
        psi3=0.80
        psi4=0.80
        gam0=0.215
        gamc0=0.215
        chi=117.4
        beta=1.0051
        siga=0.30
        rhoa=0.335
        rhou=0.0
        sigu=0.407
        alpha=0.358
        hs=1.67
        omega=-0.246
    
        
do while((abs(epsilon)>0.001) .OR. (abs(epsilon2)>0.001))
        
    call transmatrix
    

    !Initializing the value function
    
    temp3=-9999999.0
    temp3o=-9999999.0
    gn1=0.0
    gk1=0.0
    gb=0.0
    col=0.0
    c1=0.0
    gn2=0.0
    gk2=0.0
    c2=0.0
    gn3=0.0
    gk3=0.0
    c3=0.0
    gn4=0.0
    gk4=0.0
    c4=0.0
    gh4=0.0
    gn5=0.0
    gk5=0.0
    c5=0.0
    gh5=0.0
    gn6=0.0
    gk6=0.0
    c6=0.0
    gh6=0.0
    gn7=0.0
    gk7=0.0
    c7=0.0
    gn8=0.0
    gk8=0.0
    c8=0.0
    gn9=0.0
    !gk9=0.0
    c9=0.0
    gh9=0.0
    
    !Solving the problem of retired agents
    
    call max1to5
    
   temp=0
   
   do
   
   temp=temp+1
   Print *,'iteration is',temp
    
   temp2=Vone
   temp22=Vtwo
   temp23=Vthree
   temp24=Vfour
   temp25=Vfive
   temp26=Vsix
   temp27=Vseven
   temp28=Veight
   temp29=Vnine
   temp3=-9999999.0
   
   !Solving the problems of agents aged 60-20
   
   call max6
   call max7
   call max8
   call max9
   call max10
   call max11
   call max12
   
   !Checking for convergence of value functions
   
   temp3=0.0
   
    do j=1,2
            do it=1,n
                do it2=1,p
                    do it3=1,p
                        do it4=1,q
                            do it5=1,l
                            temp3(j,it,it2,it3,it4,it5)=ABS(Vthree(j,it,it2,it3,it4,it5)-temp23(j,it,it2,it3,it4,it5))
                            end do
                        end do
                    end do
                end do
            end do
    end do
   
   loc=maxloc(temp3)
   i=3
   dum3=maxval(temp3)
   
   do j=1,2
            do it=1,n
                do it2=1,p
                    do it3=1,p
                        do it4=1,q
                            do it5=1,l
                            temp3(j,it,it2,it3,it4,it5)=ABS(Vfour(j,it,it2,it3,it4,it5)-temp24(j,it,it2,it3,it4,it5))
                            end do
                        end do
                    end do
                end do
            end do
    end do
   
   if(maxval(temp3)>dum3) then
        loc=maxloc(temp3)
        i=4
        dum3=maxval(temp3)
   end if
   
   do j=1,2
            do it=1,n
                do it2=1,p
                    do it3=1,p
                        do it4=1,q
                            do it5=1,l
                            temp3(j,it,it2,it3,it4,it5)=ABS(Vfive(j,it,it2,it3,it4,it5)-temp25(j,it,it2,it3,it4,it5))
                            end do
                        end do
                    end do
                end do
            end do
    end do
   
   if(maxval(temp3)>dum3) then
        loc=maxloc(temp3)
        i=5
        dum3=maxval(temp3)
   end if
   
   do j=1,2
            do it=1,n
                do it2=1,p
                    do it3=1,p
                        do it4=1,q
                            do it5=1,l
                            temp3(j,it,it2,it3,it4,it5)=ABS(Vsix(j,it,it2,it3,it4,it5)-temp26(j,it,it2,it3,it4,it5))
                            end do
                        end do
                    end do
                end do
            end do
    end do
   
   if(maxval(temp3)>dum3) then
        loc=maxloc(temp3)
        i=6
        dum3=maxval(temp3)
   end if
   
   do j=1,2
            do it=1,n
                do it2=1,p
                    do it3=1,p
                        do it4=1,q
                            do it5=1,l
                            temp3(j,it,it2,it3,it4,it5)=ABS(Vseven(j,it,it2,it3,it4,it5)-temp27(j,it,it2,it3,it4,it5))
                            end do
                        end do
                    end do
                end do
            end do
    end do
   
   if(maxval(temp3)>dum3) then
        loc=maxloc(temp3)
        i=7
        dum3=maxval(temp3)
   end if
   
   do j=1,2
            do it=1,n
                do it2=1,p
                    do it3=1,p
                        do it4=1,q
                            do it5=1,l
                            temp3(j,it,it2,it3,it4,it5)=ABS(Veight(j,it,it2,it3,it4,it5)-temp28(j,it,it2,it3,it4,it5))
                            end do
                        end do
                    end do
                end do
            end do
    end do
   
   if(maxval(temp3)>dum3) then
        loc=maxloc(temp3)
        i=8
        dum3=maxval(temp3)
   end if
   
   do j=1,2
            do it=1,n
                do it2=1,p
                    do it3=1,p
                        do it4=1,q
                            do it5=1,l
                            temp3(j,it,it2,it3,it4,it5)=ABS(Vnine(j,it,it2,it3,it4,it5)-temp29(j,it,it2,it3,it4,it5))
                            end do
                        end do
                    end do
                end do
            end do
    end do
   
   if(maxval(temp3)>dum3) then
        loc=maxloc(temp3)
        i=9
        dum3=maxval(temp3)
   end if
   
   
   Print *,'t is',i
   Print *,'loc is',loc
   Print *,'diff is',dum3
                      
   if((dum3<0.01).OR.(temp==12)) exit
   
   end do
   
   !Simulating the model and computing various statistics
   call simulation
   call statistics
   
   epsilon2=AE-0.613183806*AEdum
   
   Print *,'AE is',AE
   Print *,'AEdum is',AEdum*0.613183806
   
   AE=0.613183806*AEdum

   Print *,'lumpsum is',lumpsum
   epsilon=abs(lumpsum-lumpsumdum)
   lumpsum=lumpsumdum
end do
   

   
   PAUSE
    
   end program IGPmodel